"""
CMasher Version
===============
Stores the different versions of the *CMasher* package.

"""


# %% VERSIONS
# Default/Latest/Current version
__version__ = "1.7.1"  # keep in sync with pyproject.toml
