# this file is generated via scripts/generate_known_cmap_types.py, do not edit manually

_CMASHER_BUILTIN_MAP_TYPES = {
    "amber": "sequential",
    "amethyst": "sequential",
    "apple": "sequential",
    "arctic": "sequential",
    "bubblegum": "sequential",
    "chroma": "sequential",
    "cmr.amber": "sequential",
    "cmr.amethyst": "sequential",
    "cmr.apple": "sequential",
    "cmr.arctic": "sequential",
    "cmr.bubblegum": "sequential",
    "cmr.chroma": "sequential",
    "cmr.copper": "cyclic",
    "cmr.cosmic": "sequential",
    "cmr.dusk": "sequential",
    "cmr.eclipse": "sequential",
    "cmr.ember": "sequential",
    "cmr.emerald": "sequential",
    "cmr.emergency": "cyclic",
    "cmr.fall": "sequential",
    "cmr.flamingo": "sequential",
    "cmr.freeze": "sequential",
    "cmr.fusion": "diverging",
    "cmr.gem": "sequential",
    "cmr.ghostlight": "sequential",
    "cmr.gothic": "sequential",
    "cmr.guppy": "diverging",
    "cmr.holly": "diverging",
    "cmr.horizon": "sequential",
    "cmr.iceburn": "diverging",
    "cmr.infinity": "cyclic",
    "cmr.jungle": "sequential",
    "cmr.lavender": "sequential",
    "cmr.lilac": "sequential",
    "cmr.neon": "sequential",
    "cmr.neutral": "sequential",
    "cmr.nuclear": "sequential",
    "cmr.ocean": "sequential",
    "cmr.pepper": "sequential",
    "cmr.pride": "diverging",
    "cmr.prinsenvlag": "diverging",
    "cmr.rainforest": "sequential",
    "cmr.redshift": "diverging",
    "cmr.sapphire": "sequential",
    "cmr.savanna": "sequential",
    "cmr.seasons": "cyclic",
    "cmr.seaweed": "diverging",
    "cmr.sepia": "sequential",
    "cmr.sunburst": "sequential",
    "cmr.swamp": "sequential",
    "cmr.torch": "sequential",
    "cmr.toxic": "sequential",
    "cmr.tree": "sequential",
    "cmr.tropical": "sequential",
    "cmr.viola": "diverging",
    "cmr.voltage": "sequential",
    "cmr.waterlily": "diverging",
    "cmr.watermelon": "diverging",
    "cmr.wildfire": "diverging",
    "copper": "cyclic",
    "cosmic": "sequential",
    "dusk": "sequential",
    "eclipse": "sequential",
    "ember": "sequential",
    "emerald": "sequential",
    "emergency": "cyclic",
    "fall": "sequential",
    "flamingo": "sequential",
    "freeze": "sequential",
    "fusion": "diverging",
    "gem": "sequential",
    "ghostlight": "sequential",
    "gothic": "sequential",
    "guppy": "diverging",
    "holly": "diverging",
    "horizon": "sequential",
    "iceburn": "diverging",
    "infinity": "cyclic",
    "jungle": "sequential",
    "lavender": "sequential",
    "lilac": "sequential",
    "neon": "sequential",
    "neutral": "sequential",
    "nuclear": "sequential",
    "ocean": "sequential",
    "pepper": "sequential",
    "pride": "diverging",
    "prinsenvlag": "diverging",
    "rainforest": "sequential",
    "redshift": "diverging",
    "sapphire": "sequential",
    "savanna": "sequential",
    "seasons": "cyclic",
    "seaweed": "diverging",
    "sepia": "sequential",
    "sunburst": "sequential",
    "swamp": "sequential",
    "torch": "sequential",
    "toxic": "sequential",
    "tree": "sequential",
    "tropical": "sequential",
    "viola": "diverging",
    "voltage": "sequential",
    "waterlily": "diverging",
    "watermelon": "diverging",
    "wildfire": "diverging",
}
